"""empty message

Revision ID: 366b5d5a835
Revises: 3d35f833a24
Create Date: 2014-07-04 18:55:01.969566

"""

# revision identifiers, used by Alembic.
revision = '366b5d5a835'
down_revision = '3d35f833a24'

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.alter_column('modifier', '_type',
               existing_type=sa.VARCHAR(length=20),
               nullable=False)
    op.alter_column('transformerref', 'transformer',
               existing_type=postgresql.BYTEA(),
               nullable=True)
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.alter_column('transformerref', 'transformer',
               existing_type=postgresql.BYTEA(),
               nullable=False)
    op.alter_column('modifier', '_type',
               existing_type=sa.VARCHAR(length=20),
               nullable=True)
    ### end Alembic commands ###
